@@ -403,16 +403,49 @@ back_button_clicked_cb (GtkButton *button,
     font_view_application_do_overview (self);
 }
 
+static void
+font_view_show_font_error (FontViewApplication *self,
+                           const gchar *message)
+{
+    GtkWidget *dialog;
+
+    dialog = gtk_message_dialog_new (GTK_WINDOW (self->main_window),
+                                     GTK_DIALOG_MODAL,
+                                     GTK_MESSAGE_ERROR,
+                                     GTK_BUTTONS_CLOSE,
+                                     _("This font could not be displayed."));
+    gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+                                              "%s",
+                                              message);
+    g_signal_connect (dialog, "response", G_CALLBACK (gtk_widget_destroy), NULL);
+    gtk_widget_show (dialog);
+}
+
+static void
+font_widget_error_cb (SushiFontWidget *font_widget,
+                      const gchar *message,
+                      gpointer user_data)
+{
+    FontViewApplication *self = user_data;
+
+    font_view_application_do_overview (self);
+    font_view_show_font_error (self, message);
+}
+
 static void
 font_widget_loaded_cb (SushiFontWidget *font_widget,
                        gpointer user_data)
 {
     FontViewApplication *self = user_data;
     FT_Face face = sushi_font_widget_get_ft_face (font_widget);
+    const gchar *uri;
 
     if (face == NULL)
         return;
 
+    uri = sushi_font_widget_get_uri (font_widget);
+    self->font_file = g_file_new_for_uri (uri);
+
     gd_main_toolbar_set_labels (GD_MAIN_TOOLBAR (self->toolbar),
                                 face->family_name, face->style_name);
     install_button_refresh_appearance (self, NULL);
@@ -453,10 +486,24 @@ info_button_clicked_cb (GtkButton *button,
 }
 
 static void
-font_view_application_do_open (FontViewApplication *self)
+font_view_ensure_model (FontViewApplication *self)
+{
+    if (self->model != NULL)
+        return;
+
+    self->model = font_view_model_new ();
+    g_signal_connect (self->model, "config-changed",
+                      G_CALLBACK (font_model_config_changed_cb), self);
+}
+
+static void
+font_view_application_do_open (FontViewApplication *self,
+                               GFile *file)
 {
     gchar *uri;
 
+    font_view_ensure_model (self);
+
     self->info_button = gd_main_toolbar_add_button (GD_MAIN_TOOLBAR (self->toolbar),
                                                     NULL, _("Info"), 
                                                     FALSE);
@@ -478,7 +525,7 @@ font_view_application_do_open (FontViewApplication *self)
 
     gtk_widget_set_vexpand (self->toolbar, FALSE);
 
-    uri = g_file_get_uri (self->font_file);
+    uri = g_file_get_uri (file);
 
     if (self->font_widget == NULL) {
         GdkRGBA white = { 1.0, 1.0, 1.0, 1.0 };
@@ -498,6 +545,8 @@ font_view_application_do_open (FontViewApplication *self)
 
         g_signal_connect (self->font_widget, "loaded",
                           G_CALLBACK (font_widget_loaded_cb), self);
+        g_signal_connect (self->font_widget, "error",
+                          G_CALLBACK (font_widget_error_cb), self);
     } else {
         g_object_set (self->font_widget, "uri", uri, NULL);
     }
@@ -517,6 +566,7 @@ icon_view_release_cb (GtkWidget *widget,
     GtkTreePath *path;
     GtkTreeIter iter;
     gchar *font_path;
+    GFile *file;
 
     /* eat double/triple click events */
     if (event->type != GDK_BUTTON_RELEASE)
@@ -532,8 +582,9 @@ icon_view_release_cb (GtkWidget *widget,
                             -1);
 
         if (font_path != NULL) {
-            self->font_file = g_file_new_for_path (font_path);
-            font_view_application_do_open (self);
+            file = g_file_new_for_path (font_path);
+            font_view_application_do_open (self, file);
+            g_object_unref (file);
         }
         gtk_tree_path_free (path);
         g_free (font_path);
@@ -562,6 +613,8 @@ font_view_application_do_overview (FontViewApplication *self)
         self->install_button = NULL;
     }
 
+    font_view_ensure_model (self);
+
     gd_main_toolbar_set_labels (GD_MAIN_TOOLBAR (self->toolbar), _("All Fonts"), NULL);
 
     if (self->icon_view == NULL) {
@@ -621,6 +674,27 @@ font_view_window_key_press_event_cb (GtkWidget *widget,
     return FALSE;
 }
 
+static void
+query_info_ready_cb (GObject *object,
+                     GAsyncResult *res,
+                     gpointer user_data)
+{
+    FontViewApplication *self = user_data;
+    GFileInfo *info;
+    GError *error = NULL;
+
+    info = g_file_query_info_finish (G_FILE (object), res, &error);
+    if (error != NULL) {
+        font_view_application_do_overview (self);
+        font_view_show_font_error (self, error->message);
+        g_error_free (error);
+    } else {
+        font_view_application_do_open (self, G_FILE (object));
+    }
+
+    g_clear_object (&info);
+}
+
 static void
 font_view_application_open (GApplication *application,
                             GFile **files,
@@ -628,9 +702,10 @@ font_view_application_open (GApplication *application,
                             const gchar *hint)
 {
     FontViewApplication *self = FONT_VIEW_APPLICATION (application);
-
-    self->font_file = g_object_ref (files[0]);
-    font_view_application_do_open (self);
+    g_file_query_info_async (files[0], G_FILE_ATTRIBUTE_STANDARD_NAME,
+                             G_FILE_QUERY_INFO_NONE,
+                             G_PRIORITY_DEFAULT, NULL,
+                             query_info_ready_cb, self);
 }
 
 static void
@@ -692,10 +767,6 @@ font_view_application_startup (GApplication *application)
     g_object_unref (builder);
     g_object_unref (menu);
 
-    self->model = font_view_model_new ();
-    g_signal_connect (self->model, "config-changed",
-                      G_CALLBACK (font_model_config_changed_cb), self);
-
     self->main_window = window = gtk_application_window_new (GTK_APPLICATION (application));
     gtk_window_set_resizable (GTK_WINDOW (window), TRUE);
     gtk_window_set_default_size (GTK_WINDOW (window), 800, 600);
